LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use IEEE.std_logic_textio.all;
use STD.textio.all;
use IEEE.numeric_std.all;
use ieee.math_real.all;
 
ENTITY artix_ftdi_test_bench IS
END artix_ftdi_test_bench;
 
ARCHITECTURE behavior OF artix_ftdi_test_bench IS 
 
COMPONENT artix_ftdi_test PORT(
	CLK : IN  std_logic;
	D : INOUT  std_logic_vector(7 downto 0);
	RXF : IN  std_logic;
	TXE : IN  std_logic;
	RD : OUT  std_logic;
	WR : OUT  std_logic;
	SIWU : OUT  std_logic;
	CLKOUT : IN  std_logic;
	OE : OUT  std_logic;
	PWRSAV : OUT  std_logic;
	sysled : OUT  std_logic);
END COMPONENT;
    
--Inputs
signal CLK : std_logic := '0';
signal RXF : std_logic := '0';
signal TXE : std_logic := '0';
signal CLKOUT : std_logic := '0';

--BiDirs
signal D : std_logic_vector(7 downto 0);

--Outputs
signal RD : std_logic;
signal WR : std_logic;
signal SIWU : std_logic;
signal OE : std_logic;
signal PWRSAV : std_logic;
signal sysled : std_logic;

-- Clock period definitions
constant CLK_period : time := 10 ns;
constant CLKOUT_period : time := 16.66 ns;

signal rand_num_1k : integer := 0;
signal rand_num_715ps : integer := 0;
signal txe_counter: integer range 0 to 127:=0;
signal D_old_u, D_u : unsigned(7 downto 0):=(others => '0');
 
BEGIN
 
-- Instantiate the Unit Under Test (UUT)
uut: artix_ftdi_test PORT MAP(
	CLK => CLK,
	D => D,
	RXF => RXF,
	TXE => TXE,
	RD => RD,
	WR => WR,
	SIWU => SIWU,
	CLKOUT => CLKOUT,
	OE => OE,
	PWRSAV => PWRSAV,
	sysled => sysled);

process
    variable seed1, seed2: positive;               -- seed values for random generator
    variable rand: real;   -- random real-number value in range 0 to 1.0  
	begin
    uniform(seed1, seed2, rand);   -- generate random number
    rand_num_1k <= integer(rand*1000.0);  -- rescale to 0..1000, convert integer part
	rand_num_715ps <= 1+integer(rand*6.15);
    wait for 100 ns;
end process;
	
CLK_process :process begin
	CLK <= '0';
	wait for CLK_period/2;
	CLK <= '1';
	wait for CLK_period/2;
	end process;
 
CLKOUT_process :process begin
	CLKOUT <= '0';
	wait for CLKOUT_period/2;
	CLKOUT <= '1';
	wait for CLKOUT_period/2;
	end process;
 
stim_proc: process begin
	wait until rising_edge(CLKOUT);
	txe_counter <= txe_counter +1;
	if txe_counter = 127 then
		wait for rand_num_715ps*1ns;
		TXE <= '1';
		wait for rand_num_1k*1ns;
		wait until rising_edge(CLKOUT);
		wait for rand_num_715ps*1ns;
		TXE <= '0';
		txe_counter <= 0;
	end if;
	end process;
   
write_io: process
	file my_output : TEXT open WRITE_MODE is "file_io.txt";
	variable my_line : LINE;
	variable whatsTheTime : time;
begin		
	wait until rising_edge(CLKOUT);
	if WR = '0' and TXE = '0' then
		D_old_u <= D_u;
		D_u <= unsigned(D);
		if unsigned(D) /= D_u +1 and D_u /= 0 then
			whatsTheTime := now;
			write(my_line,string'("FAIL - "));
			write(my_line,to_integer(D_old_u));
			write(my_line,string'(" - "));
			write(my_line,to_integer(D_u));
			write(my_line,string'(" - "));
			write(my_line,whatsTheTime);
			writeline(output,my_line);
		end if;
	end if;

	end process;
END;